#include "types.h"


// CRC-8 look-up table for PEC byte calculation
BYTE code crc8_table[256] = {
    0x00, // entry 0x00
    0x07, // entry 0x01
    0x0E, // entry 0x02
    0x09, // entry 0x03
    0x1C, // entry 0x04
    0x1B, // entry 0x05
    0x12, // entry 0x06
    0x15, // entry 0x07
    0x38, // entry 0x08
    0x3F, // entry 0x09
    0x36, // entry 0x0A
    0x31, // entry 0x0B
    0x24, // entry 0x0C
    0x23, // entry 0x0D
    0x2A, // entry 0x0E
    0x2D, // entry 0x0F
    0x70, // entry 0x10
    0x77, // entry 0x11
    0x7E, // entry 0x12
    0x79, // entry 0x13
    0x6C, // entry 0x14
    0x6B, // entry 0x15
    0x62, // entry 0x16
    0x65, // entry 0x17
    0x48, // entry 0x18
    0x4F, // entry 0x19
    0x46, // entry 0x1A
    0x41, // entry 0x1B
    0x54, // entry 0x1C
    0x53, // entry 0x1D
    0x5A, // entry 0x1E
    0x5D, // entry 0x1F
    0xE0, // entry 0x20
    0xE7, // entry 0x21
    0xEE, // entry 0x22
    0xE9, // entry 0x23
    0xFC, // entry 0x24
    0xFB, // entry 0x25
    0xF2, // entry 0x26
    0xF5, // entry 0x27
    0xD8, // entry 0x28
    0xDF, // entry 0x29
    0xD6, // entry 0x2A
    0xD1, // entry 0x2B
    0xC4, // entry 0x2C
    0xC3, // entry 0x2D
    0xCA, // entry 0x2E
    0xCD, // entry 0x2F
    0x90, // entry 0x30
    0x97, // entry 0x31
    0x9E, // entry 0x32
    0x99, // entry 0x33
    0x8C, // entry 0x34
    0x8B, // entry 0x35
    0x82, // entry 0x36
    0x85, // entry 0x37
    0xA8, // entry 0x38
    0xAF, // entry 0x39
    0xA6, // entry 0x3A
    0xA1, // entry 0x3B
    0xB4, // entry 0x3C
    0xB3, // entry 0x3D
    0xBA, // entry 0x3E
    0xBD, // entry 0x3F
    0xC7, // entry 0x40
    0xC0, // entry 0x41
    0xC9, // entry 0x42
    0xCE, // entry 0x43
    0xDB, // entry 0x44
    0xDC, // entry 0x45
    0xD5, // entry 0x46
    0xD2, // entry 0x47
    0xFF, // entry 0x48
    0xF8, // entry 0x49
    0xF1, // entry 0x4A
    0xF6, // entry 0x4B
    0xE3, // entry 0x4C
    0xE4, // entry 0x4D
    0xED, // entry 0x4E
    0xEA, // entry 0x4F
    0xB7, // entry 0x50
    0xB0, // entry 0x51
    0xB9, // entry 0x52
    0xBE, // entry 0x53
    0xAB, // entry 0x54
    0xAC, // entry 0x55
    0xA5, // entry 0x56
    0xA2, // entry 0x57
    0x8F, // entry 0x58
    0x88, // entry 0x59
    0x81, // entry 0x5A
    0x86, // entry 0x5B
    0x93, // entry 0x5C
    0x94, // entry 0x5D
    0x9D, // entry 0x5E
    0x9A, // entry 0x5F
    0x27, // entry 0x60
    0x20, // entry 0x61
    0x29, // entry 0x62
    0x2E, // entry 0x63
    0x3B, // entry 0x64
    0x3C, // entry 0x65
    0x35, // entry 0x66
    0x32, // entry 0x67
    0x1F, // entry 0x68
    0x18, // entry 0x69
    0x11, // entry 0x6A
    0x16, // entry 0x6B
    0x03, // entry 0x6C
    0x04, // entry 0x6D
    0x0D, // entry 0x6E
    0x0A, // entry 0x6F
    0x57, // entry 0x70
    0x50, // entry 0x71
    0x59, // entry 0x72
    0x5E, // entry 0x73
    0x4B, // entry 0x74
    0x4C, // entry 0x75
    0x45, // entry 0x76
    0x42, // entry 0x77
    0x6F, // entry 0x78
    0x68, // entry 0x79
    0x61, // entry 0x7A
    0x66, // entry 0x7B
    0x73, // entry 0x7C
    0x74, // entry 0x7D
    0x7D, // entry 0x7E
    0x7A, // entry 0x7F
    0x89, // entry 0x80
    0x8E, // entry 0x81
    0x87, // entry 0x82
    0x80, // entry 0x83
    0x95, // entry 0x84
    0x92, // entry 0x85
    0x9B, // entry 0x86
    0x9C, // entry 0x87
    0xB1, // entry 0x88
    0xB6, // entry 0x89
    0xBF, // entry 0x8A
    0xB8, // entry 0x8B
    0xAD, // entry 0x8C
    0xAA, // entry 0x8D
    0xA3, // entry 0x8E
    0xA4, // entry 0x8F
    0xF9, // entry 0x90
    0xFE, // entry 0x91
    0xF7, // entry 0x92
    0xF0, // entry 0x93
    0xE5, // entry 0x94
    0xE2, // entry 0x95
    0xEB, // entry 0x96
    0xEC, // entry 0x97
    0xC1, // entry 0x98
    0xC6, // entry 0x99
    0xCF, // entry 0x9A
    0xC8, // entry 0x9B
    0xDD, // entry 0x9C
    0xDA, // entry 0x9D
    0xD3, // entry 0x9E
    0xD4, // entry 0x9F
    0x69, // entry 0xA0
    0x6E, // entry 0xA1
    0x67, // entry 0xA2
    0x60, // entry 0xA3
    0x75, // entry 0xA4
    0x72, // entry 0xA5
    0x7B, // entry 0xA6
    0x7C, // entry 0xA7
    0x51, // entry 0xA8
    0x56, // entry 0xA9
    0x5F, // entry 0xAA
    0x58, // entry 0xAB
    0x4D, // entry 0xAC
    0x4A, // entry 0xAD
    0x43, // entry 0xAE
    0x44, // entry 0xAF
    0x19, // entry 0xB0
    0x1E, // entry 0xB1
    0x17, // entry 0xB2
    0x10, // entry 0xB3
    0x05, // entry 0xB4
    0x02, // entry 0xB5
    0x0B, // entry 0xB6
    0x0C, // entry 0xB7
    0x21, // entry 0xB8
    0x26, // entry 0xB9
    0x2F, // entry 0xBA
    0x28, // entry 0xBB
    0x3D, // entry 0xBC
    0x3A, // entry 0xBD
    0x33, // entry 0xBE
    0x34, // entry 0xBF
    0x4E, // entry 0xC0
    0x49, // entry 0xC1
    0x40, // entry 0xC2
    0x47, // entry 0xC3
    0x52, // entry 0xC4
    0x55, // entry 0xC5
    0x5C, // entry 0xC6
    0x5B, // entry 0xC7
    0x76, // entry 0xC8
    0x71, // entry 0xC9
    0x78, // entry 0xCA
    0x7F, // entry 0xCB
    0x6A, // entry 0xCC
    0x6D, // entry 0xCD
    0x64, // entry 0xCE
    0x63, // entry 0xCF
    0x3E, // entry 0xD0
    0x39, // entry 0xD1
    0x30, // entry 0xD2
    0x37, // entry 0xD3
    0x22, // entry 0xD4
    0x25, // entry 0xD5
    0x2C, // entry 0xD6
    0x2B, // entry 0xD7
    0x06, // entry 0xD8
    0x01, // entry 0xD9
    0x08, // entry 0xDA
    0x0F, // entry 0xDB
    0x1A, // entry 0xDC
    0x1D, // entry 0xDD
    0x14, // entry 0xDE
    0x13, // entry 0xDF
    0xAE, // entry 0xE0
    0xA9, // entry 0xE1
    0xA0, // entry 0xE2
    0xA7, // entry 0xE3
    0xB2, // entry 0xE4
    0xB5, // entry 0xE5
    0xBC, // entry 0xE6
    0xBB, // entry 0xE7
    0x96, // entry 0xE8
    0x91, // entry 0xE9
    0x98, // entry 0xEA
    0x9F, // entry 0xEB
    0x8A, // entry 0xEC
    0x8D, // entry 0xED
    0x84, // entry 0xEE
    0x83, // entry 0xEF
    0xDE, // entry 0xF0
    0xD9, // entry 0xF1
    0xD0, // entry 0xF2
    0xD7, // entry 0xF3
    0xC2, // entry 0xF4
    0xC5, // entry 0xF5
    0xCC, // entry 0xF6
    0xCB, // entry 0xF7
    0xE6, // entry 0xF8
    0xE1, // entry 0xF9
    0xE8, // entry 0xFA
    0xEF, // entry 0xFB
    0xFA, // entry 0xFC
    0xFD, // entry 0xFD
    0xF4, // entry 0xFE
    0xF3  // entry 0xFF
};


BYTE calc_pec(BYTE start_crc8, BYTE length, BYTE* start_addr);


// function: calculate PEC byte based on a 256-entry CRC-8 look-up table
// entries: start_addr - starting address of memory, based on which a PEC byte will be calculated
//          start_crc8 - the starting CRC-8 value
//          length   - # of bytes to calculate on
// return: an 8-bit CRC-8 that will be used as the PEC byte
BYTE calc_pec(BYTE start_crc8, BYTE length, BYTE* start_addr)
{
    BYTE i = 0;
    BYTE crc8 = start_crc8;

    for (; i < length; i++)
    {
        crc8 ^= *(start_addr+i); // XOR the previous CRC-8 result with the new byte of data
        crc8 = crc8_table[crc8];
    }

    return crc8;
}

